'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var prefixAll = _interopDefault(require('inline-style-prefixer/static'));
var test = _interopDefault(require('tape'));

const uppercasePattern = /[A-Z]/g;
const msPattern = /^ms-/;
const cache = {};

function hyphenateStyleName(prop) {
  return prop in cache ? cache[prop] : cache[prop] = prop.replace(uppercasePattern, '-$&').toLowerCase().replace(msPattern, '-ms-');
}

function injectStyle(styletron, styles, media, pseudo) {
  let classString = '';
  for (const key in styles) {
    const val = styles[key];
    const valType = typeof val;
    if (valType === 'string' || valType === 'number') {
      classString += ' ' + styletron.injectRawDeclaration({
        block: `${hyphenateStyleName(key)}:${val}`,
        media,
        pseudo
      });
      continue;
    }
    if (Array.isArray(val)) {
      if (val.length === 0) {
        continue;
      }
      const hyphenated = hyphenateStyleName(key);
      let block = `${hyphenated}:${val[0]}`;
      for (let i = 1; i < val.length; i++) {
        block += `;${hyphenated}:${val[i]}`;
      }
      classString += ' ' + styletron.injectRawDeclaration({
        block,
        media,
        pseudo
      });
      continue;
    }
    if (valType === 'object') {
      if (key[0] === ':') {
        classString += ' ' + injectStyle(styletron, val, media, key);
        continue;
      }
      if (key.substring(0, 6) === '@media') {
        classString += ' ' + injectStyle(styletron, val, key.substr(7), pseudo);
        continue;
      }
    }
  }
  // remove leading space on way out
  return classString.slice(1);
}

const prefixedBlockCache = {};

function injectStylePrefixed(styletron, styles, media, pseudo, cache = prefixedBlockCache) {
  let classString = '';
  for (const originalKey in styles) {
    const originalVal = styles[originalKey];
    const originalValType = typeof originalVal;
    const isPrimitiveVal = originalValType === 'string' || originalValType === 'number';
    if (isPrimitiveVal || Array.isArray(originalVal)) {
      let block = '';
      if (isPrimitiveVal && cache.hasOwnProperty(originalKey) && cache[originalKey].hasOwnProperty(originalVal)) {
        block = cache[originalKey][originalVal];
      } else {
        const prefixed = prefixAll({ [originalKey]: originalVal });
        for (const prefixedKey in prefixed) {
          const prefixedVal = prefixed[prefixedKey];
          const prefixedValType = typeof prefixedVal;
          if (prefixedValType === 'string' || prefixedValType === 'number') {
            block += `${hyphenateStyleName(prefixedKey)}:${prefixedVal};`;
            continue;
          }
          if (Array.isArray(prefixedVal)) {
            const hyphenated = hyphenateStyleName(prefixedKey);
            for (let i = 0; i < prefixedVal.length; i++) {
              block += `${hyphenated}:${prefixedVal[i]};`;
            }
            continue;
          }
        }
        block = block.slice(0, -1); // Remove trailing semicolon
        if (isPrimitiveVal) {
          if (!cache.hasOwnProperty(originalKey)) {
            cache[originalKey] = {};
          }
          cache[originalKey][originalVal] = block;
        }
      }
      classString += ' ' + styletron.injectRawDeclaration({
        block,
        media,
        pseudo
      });
    }
    if (originalValType === 'object') {
      if (originalKey[0] === ':') {
        classString += ' ' + injectStylePrefixed(styletron, originalVal, media, originalKey, cache);
        continue;
      }
      if (originalKey.substring(0, 6) === '@media') {
        classString += ' ' + injectStylePrefixed(styletron, originalVal, originalKey.substr(7), pseudo, cache);
        continue;
      }
    }
  }
  // remove leading space on way out
  return classString.slice(1);
}

test('test injection', t => {
  const decls = [];
  const spy = {
    injectRawDeclaration: decl => {
      decls.push(decl);
      return decls.length;
    }
  };
  const classString = injectStyle(spy, {
    color: 'red',
    backgroundColor: 'blue',
    '@media (max-width: 500px)': {
      color: 'purple'
    },
    ':hover': {
      background: 'orange'
    }
  });
  t.equal(classString, '1 2 3 4');
  t.deepEqual(decls, [{ block: 'color:red', media: undefined, pseudo: undefined }, {
    block: 'background-color:blue',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'color:purple',
    media: '(max-width: 500px)',
    pseudo: undefined
  }, { block: 'background:orange', media: undefined, pseudo: ':hover' }]);
  t.end();
});

test('test injection array', function (t) {
  const decls = [];
  const spy = {
    injectRawDeclaration: function injectRawDeclaration(decl) {
      decls.push(decl);
      return decls.length;
    }
  };
  const classString = injectStyle(spy, {
    color: ['red', 'blue'],
    '@media (max-width: 500px)': {
      color: ['purple', 'orange']
    },
    ':hover': {
      color: ['green', 'yellow']
    }
  });
  t.equal(classString, '1 2 3');
  t.deepEqual(decls, [{ block: 'color:red;color:blue', media: undefined, pseudo: undefined }, {
    block: 'color:purple;color:orange',
    media: '(max-width: 500px)',
    pseudo: undefined
  }, { block: 'color:green;color:yellow', media: undefined, pseudo: ':hover' }]);
  t.end();
});

test('test injection prefixed', function (t) {
  const decls = [];
  const spy = {
    injectRawDeclaration: function injectRawDeclaration(decl) {
      decls.push(decl);
      return decls.length;
    }
  };
  const classString = injectStylePrefixed(spy, {
    display: 'flex',
    height: ['min-content', '100%'],
    transition: 'height 1s',
    ':hover': {
      backgroundColor: 'linear-gradient(to bottom, red, green)'
    },
    '@media (max-width: 500px)': {
      flexGrow: 1
    }
  });
  t.equal(classString, '1 2 3 4 5');
  t.deepEqual(decls, [{
    block: 'display:-webkit-box;display:-moz-box;display:-ms-flexbox;display:-webkit-flex;display:flex',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'height:-webkit-min-content;height:-moz-min-content;height:min-content;height:100%',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'transition:height 1s;-webkit-transition:height 1s;-moz-transition:height 1s',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'background-color:-webkit-linear-gradient(to bottom, red, green);background-color:-moz-linear-gradient(to bottom, red, green);background-color:linear-gradient(to bottom, red, green)',
    media: undefined,
    pseudo: ':hover'
  }, {
    block: 'flex-grow:1;-webkit-flex-grow:1',
    media: '(max-width: 500px)',
    pseudo: undefined
  }]);
  t.end();
});

test('test prefixed cache', function (t) {
  const decls = [];
  const spy = {
    injectRawDeclaration: function injectRawDeclaration(decl) {
      decls.push(decl);
      return decls.length;
    }
  };
  const cache = { flexGrow: { 1: 'color:red' } };
  injectStylePrefixed(spy, {
    display: 'flex',
    flexGrow: 1
  }, undefined, undefined, cache);
  t.deepEqual(cache, {
    display: {
      flex: 'display:-webkit-box;display:-moz-box;display:-ms-flexbox;display:-webkit-flex;display:flex'
    },
    flexGrow: { 1: 'color:red' }
  });
  t.deepEqual(decls, [{
    block: 'display:-webkit-box;display:-moz-box;display:-ms-flexbox;display:-webkit-flex;display:flex',
    media: undefined,
    pseudo: undefined
  }, { block: 'color:red', media: undefined, pseudo: undefined }]);
  t.end();
});
//# sourceMappingURL=node.js.map
